@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.c
-copyright 2008-2010,2012, Thomas E. Dickey
+copyright 2008-2012,2013, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.61 2012/11/29 09:32:21 tom Exp $
+ * $MawkId: bi_funct.c,v 1.67 2013/02/19 10:57:28 tom Exp $
  * @Log: bi_funct.c,v @
  * Revision 1.9  1996/01/14  17:16:11  mike
  * flush_all_output() before system()
@@ -78,6 +78,8 @@ the GNU General Public License, version 2, 1991.
 #include <math.h>
 #include <time.h>
 
+/* #define EXP_UNROLLED_GSUB */
+
 #if OPT_TRACE > 0
 #define return_CELL(func, cell) TRACE(("..." func " ->")); \
 				TRACE_CELL(cell); \
@@ -285,9 +287,10 @@ bi_substr(CELL * sp)
     }
 
     if (n_args == 2) {
-	n = MAX__INT;
-	if (sp[1].type != C_DOUBLE)
+	n = len;
+	if (sp[1].type != C_DOUBLE) {
 	    cast1_to_d(sp + 1);
+	}
     } else {
 	if (TEST2(sp + 1) != TWO_DOUBLES)
 	    cast2_to_d(sp + 1);
@@ -295,6 +298,14 @@ bi_substr(CELL * sp)
     }
     i = d_to_i(sp[1].dval) - 1;	/* i now indexes into string */
 
+    /*
+     * If the starting index is past the end of the string, there is nothing
+     * to extract other than an empty string.
+     */
+    if (i > len) {
+	n = 0;
+    }
+
     /*
      * Workaround in case someone's written a script that does substr(0,last-1)
      * by transforming it into substr(1,last).
@@ -303,6 +314,10 @@ bi_substr(CELL * sp)
 	n -= i + 1;
 	i = 0;
     }
+
+    /*
+     * Keep 'n' from extending past the end of the string. 
+     */
     if (n > len - i) {
 	n = len - i;
     }
@@ -1177,6 +1192,7 @@ typedef struct {
 #define NextTarget     NextGSUB.target
 #define NextTargetLen  NextGSUB.target_len
 
+#ifdef EXP_UNROLLED_GSUB
 /* #define DEBUG_GSUB 1 */
 
 static size_t gsub_max;
@@ -1523,3 +1539,142 @@ bi_gsub(CELL * sp)
 
     return_CELL("bi_gsub", sp);
 }
+
+#else /* GSUB uses stack... */
+static unsigned repl_cnt;	/* number of global replacements */
+
+/* recursive global subsitution
+   dealing with empty matches makes this mildly painful
+
+   repl is always of type REPL or REPLV, destroyed by caller
+   flag is set if, match of empty string at front is OK
+*/
+
+static STRING *
+gsub(PTR re, CELL * repl, char *target, size_t target_len, int flag)
+{
+    char *front = 0, *middle;
+    STRING *back;
+    size_t front_len, middle_len;
+    STRING *ret_val;
+    CELL xrepl;			/* a copy of repl so we can change repl */
+
+    if (!(middle = REmatch(target, target_len, cast_to_re(re), &middle_len)))
+	return new_STRING(target);	/* no match */
+
+    cellcpy(&xrepl, repl);
+
+    if (!flag && middle_len == 0 && middle == target) {
+	/* match at front that's not allowed */
+
+	if (*target == 0) {	/* target is empty string */
+	    repl_destroy(&xrepl);
+	    null_str.ref_cnt++;
+	    return &null_str;
+	} else if (1 && isAnchored(re)) {
+	    repl_destroy(&xrepl);
+	    return new_STRING1(target, target_len);
+	} else {
+	    char xbuff[2];
+
+	    front_len = 0;
+	    /* make new repl with target[0] */
+	    repl_destroy(repl);
+	    --target_len;
+	    xbuff[0] = *target++;
+	    xbuff[1] = 0;
+	    repl->type = C_REPL;
+	    repl->ptr = (PTR) new_STRING(xbuff);
+	    back = gsub(re, &xrepl, target, target_len, 1);
+	}
+    } else {			/* a match that counts */
+	repl_cnt++;
+
+	front = target;
+	front_len = (unsigned) (middle - target);
+
+	if (front_len == target_len) {	/* matched back of target */
+	    back = &null_str;
+	    null_str.ref_cnt++;
+	} else {
+	    back = gsub(re,
+			&xrepl,
+			middle + middle_len,
+			target_len - (front_len + middle_len),
+			0);
+	}
+
+	/* patch the &'s if needed */
+	if (repl->type == C_REPLV) {
+	    STRING *sval = new_STRING0(middle_len);
+
+	    memcpy(sval->str, middle, middle_len);
+	    replv_to_repl(repl, sval);
+	    free_STRING(sval);
+	}
+    }
+
+    /* put the three pieces together */
+    ret_val = new_STRING0(front_len + string(repl)->len + back->len);
+    {
+	char *p = ret_val->str;
+
+	if (front_len) {
+	    memcpy(p, front, front_len);
+	    p += front_len;
+	}
+
+	if (string(repl)->len) {
+	    memcpy(p, string(repl)->str, string(repl)->len);
+	    p += string(repl)->len;
+	}
+	if (back->len)
+	    memcpy(p, back->str, back->len);
+    }
+
+    /* cleanup, repl is freed by the caller */
+    repl_destroy(&xrepl);
+    free_STRING(back);
+
+    return ret_val;
+}
+
+/* set up for call to gsub() */
+CELL *
+bi_gsub(CELL * sp)
+{
+    CELL *cp;			/* pts at the replacement target */
+    CELL sc;			/* copy of replacement target */
+    CELL tc;			/* build the result here */
+
+    TRACE_FUNC("bi_gsub", sp);
+
+    sp -= 2;
+    if (sp->type != C_RE)
+	cast_to_RE(sp);
+    if ((sp + 1)->type != C_REPL && (sp + 1)->type != C_REPLV)
+	cast_to_REPL(sp + 1);
+
+    cellcpy(&sc, cp = (CELL *) (sp + 2)->ptr);
+    if (sc.type < C_STRING)
+	cast1_to_s(&sc);
+
+    repl_cnt = 0;
+    tc.ptr = (PTR) gsub(sp->ptr, sp + 1, string(&sc)->str, string(&sc)->len, 1);
+
+    if (repl_cnt) {
+	tc.type = C_STRING;
+	slow_cell_assign(cp, &tc);
+    }
+
+    /* cleanup */
+    free_STRING(string(&sc));
+    free_STRING(string(&tc));
+    repl_destroy(sp + 1);
+
+    sp->type = C_DOUBLE;
+    sp->dval = (double) repl_cnt;
+
+    return_CELL("bi_gsub", sp);
+}
+#endif /* EXP_UNROLLED_GSUB */
