@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.h
-copyright 2009, Thomas E. Dickey
+copyright 2009-2010,2012 Thomas E. Dickey
 copyright 1991, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.h,v 1.3 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: bi_funct.h,v 1.5 2012/10/27 14:09:10 tom Exp $
  * @Log: bi_funct.h,v @
  * Revision 1.2  1994/12/11  22:10:15  mike
  * fflush
@@ -59,6 +59,11 @@ CELL *bi_sqrt(CELL *);
 CELL *bi_srand(CELL *);
 CELL *bi_rand(CELL *);
 
+/* time functions */
+CELL *bi_mktime(CELL *);
+CELL *bi_strftime(CELL *);
+CELL *bi_systime(CELL *);
+
 /* other builtins */
 CELL *bi_close(CELL *);
 CELL *bi_system(CELL *);
