@@ -1,6 +1,7 @@
 /********************************************
 fin.h
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 1991-1992,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fin.h,v 1.8 2009/12/16 23:44:26 tom Exp $
+ * $MawkId: fin.h,v 1.11 2010/12/10 17:00:00 tom Exp $
  * @Log: fin.h,v @
  * Revision 1.1.1.1  1993/07/03  18:58:13  mike
  * move source to cvs
@@ -50,8 +51,8 @@ FIN *FINdopen(int, int);
 FIN *FINopen(char *, int);
 void FINclose(FIN *);
 void FINsemi_close(FIN *);
-char *FINgets(FIN *, unsigned *);
-unsigned fillbuff(int, char *, unsigned);
+char *FINgets(FIN *, size_t *);
+size_t fillbuff(int, char *, size_t);
 
 extern FIN *main_fin;		/* for the main input stream */
 void open_main(void);
