@@ -1,6 +1,8 @@
 /********************************************
 rexp2.c
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 2010, Jonathan Nieder
+copyright 1991-1992,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp2.c,v 1.11 2009/09/14 09:02:53 tom Exp $
+ * $MawkId: rexp2.c,v 1.21 2010/12/10 17:00:00 tom Exp $
  * @Log: rexp2.c,v @
  * Revision 1.3  1993/07/24  17:55:12  mike
  * more cleanup
@@ -71,6 +73,10 @@ RT_STATE *RE_run_stack_limit;
    This hack fixes it without rewriting the whole thing, 5/31/91 */
 RT_STATE *RE_run_stack_empty;
 
+RT_POS_ENTRY *RE_pos_stack_base;
+RT_POS_ENTRY *RE_pos_stack_limit;
+RT_POS_ENTRY *RE_pos_stack_empty;
+
 void
 RE_run_stack_init(void)
 {
@@ -82,6 +88,21 @@ RE_run_stack_init(void)
     }
 }
 
+void
+RE_pos_stack_init(void)
+{
+    if (!RE_pos_stack_base) {
+	RE_pos_stack_base = (RT_POS_ENTRY *)
+	    RE_malloc(sizeof(RT_POS_ENTRY) * STACKGROWTH);
+	RE_pos_stack_limit = RE_pos_stack_base + STACKGROWTH;
+	RE_pos_stack_empty = RE_pos_stack_base;
+
+	RE_pos_stack_base->pos = NULL;	/* RE_pos_peek(RE_pos_stack_empty) */
+	RE_pos_stack_base->owner = -1;	/* avoid popping base */
+	RE_pos_stack_base->prev_offset = 0;	/* avoid popping below base */
+    }
+}
+
 /* sometimes during REmatch(), this stack can grow pretty large.
    In real life cases, the back tracking usually fails. Some
    work is needed here to improve the algorithm.
@@ -91,8 +112,8 @@ RE_run_stack_init(void)
 RT_STATE *
 RE_new_run_stack(void)
 {
-    unsigned oldsize = (unsigned) (RE_run_stack_limit - RE_run_stack_base);
-    unsigned newsize = oldsize + STACKGROWTH;
+    size_t oldsize = (size_t) (RE_run_stack_limit - RE_run_stack_base);
+    size_t newsize = oldsize + STACKGROWTH;
 
 #ifdef	LMDOS			/* large model DOS */
     /* have to worry about overflow on multiplication (ugh) */
@@ -109,7 +130,7 @@ RE_new_run_stack(void)
 	/* this is pretty unusual, I've only seen it happen on
 	   weird input to REmatch() under 16bit DOS , the same
 	   situation worked easily on 32bit machine.  */
-	exit(100);
+	mawk_exit(100);
     }
 
     RE_run_stack_limit = RE_run_stack_base + newsize;
@@ -119,12 +140,37 @@ RE_new_run_stack(void)
     return RE_run_stack_base + oldsize;
 }
 
+RT_POS_ENTRY *
+RE_new_pos_stack(void)
+{
+    size_t oldsize = (size_t) (RE_pos_stack_limit - RE_pos_stack_base);
+    size_t newsize = oldsize + STACKGROWTH;
+
+    /* FIXME: handle overflow on multiplication for large model DOS
+     * (see RE_new_run_stack()).
+     */
+    RE_pos_stack_base = (RT_POS_ENTRY *)
+	realloc(RE_pos_stack_base, newsize * sizeof(RT_POS_ENTRY));
+
+    if (!RE_pos_stack_base) {
+	fprintf(stderr, "out of memory for RE string position stack\n");
+	mawk_exit(100);
+    }
+
+    RE_pos_stack_limit = RE_pos_stack_base + newsize;
+    RE_pos_stack_empty = RE_pos_stack_base;
+
+    /* return the new stackp */
+    return RE_pos_stack_base + oldsize;
+}
+
 #ifdef	DEBUG
 static RT_STATE *
 slow_push(
 	     RT_STATE * sp,
 	     STATE * m,
 	     char *s,
+	     RT_POS_ENTRY * pos_top,
 	     int u)
 {
     if (sp == RE_run_stack_limit)
@@ -132,18 +178,26 @@ slow_push(
     sp->m = m;
     sp->s = s;
     sp->u = u;
+    sp->sp = pos_top - RE_pos_stack_base;
+    sp->tp = pos_top->prev_offset;
     return sp;
 }
 #endif
 
 #ifdef	 DEBUG
-#define	 push(mx,sx,ux)	  stackp = slow_push(++stackp, mx, sx, ux)
+#define	 push(mx,sx,px,ux) do { \
+		stackp = slow_push(++stackp, mx, sx, px, ux); \
+	} while(0)
 #else
-#define	 push(mx,sx,ux)	  if (++stackp == RE_run_stack_limit) \
-				stackp = RE_new_run_stack(); \
-			  stackp->m = (mx); \
-			  stackp->s = (sx); \
-			  stackp->u = (ux)
+#define	 push(mx,sx,px,ux) do { \
+		if (++stackp == RE_run_stack_limit) \
+			stackp = RE_new_run_stack(); \
+		stackp->m = (mx); \
+		stackp->s = (sx); \
+		stackp->u = (ux); \
+		stackp->sp = (int) ((px) - RE_pos_stack_base); \
+		stackp->tp = (px)->prev_offset; \
+	} while(0)
 #endif
 
 #define	  CASE_UANY(x)	case  x + U_OFF :  case	 x + U_ON
@@ -153,7 +207,7 @@ slow_push(
  */
 int
 REtest(char *str,		/* string to test */
-       unsigned len,		/* ...its length */
+       size_t len,		/* ...its length */
        PTR machine)		/* compiled regular-expression */
 {
     register STATE *m = (STATE *) machine;
@@ -161,15 +215,17 @@ REtest(char *str,		/* string to test */
     register RT_STATE *stackp;
     int u_flag;
     char *str_end = str + len;
+    RT_POS_ENTRY *sp;
     int t;			/*convenient temps */
     STATE *tm;
 
     /* handle the easy case quickly */
     if ((m + 1)->s_type == M_ACCEPT && m->s_type == M_STR) {
-	return str_str(s, len, m->s_data.str, m->s_len) != (char *) 0;
+	return str_str(s, len, m->s_data.str, (size_t) m->s_len) != (char *) 0;
     } else {
 	u_flag = U_ON;
 	stackp = RE_run_stack_empty;
+	sp = RE_pos_stack_empty;
 	goto reswitch;
     }
 
@@ -178,13 +234,15 @@ REtest(char *str,		/* string to test */
 	return 0;
     m = stackp->m;
     s = stackp->s;
+    sp = RE_pos_stack_base + stackp->sp;
+    sp->prev_offset = stackp->tp;
     u_flag = (stackp--)->u;
 
   reswitch:
 
     switch (m->s_type + u_flag) {
     case M_STR + U_OFF + END_OFF:
-	if (strncmp(s, m->s_data.str, m->s_len))
+	if (strncmp(s, m->s_data.str, (size_t) m->s_len))
 	    goto refill;
 	s += m->s_len;
 	m++;
@@ -198,17 +256,17 @@ REtest(char *str,		/* string to test */
 	goto reswitch;
 
     case M_STR + U_ON + END_OFF:
-	if (!(s = str_str(s, (unsigned) (str_end - s), m->s_data.str, m->s_len)))
+	if (!(s = str_str(s, (size_t) (str_end - s), m->s_data.str, (size_t) m->s_len)))
 	    goto refill;
-	push(m, s + 1, U_ON);
+	push(m, s + 1, sp, U_ON);
 	s += m->s_len;
 	m++;
 	u_flag = U_OFF;
 	goto reswitch;
 
     case M_STR + U_ON + END_ON:
-	t = (str_end - s) - m->s_len;
-	if (t < 0 || memcmp(s + t, m->s_data.str, m->s_len))
+	t = (int) (str_end - s) - m->s_len;
+	if (t < 0 || memcmp(s + t, m->s_data.str, (size_t) m->s_len))
 	    goto refill;
 	s = str_end;
 	m++;
@@ -240,7 +298,7 @@ REtest(char *str,		/* string to test */
 	    s++;
 	}
 	s++;
-	push(m, s, U_ON);
+	push(m, s, sp, U_ON);
 	m++;
 	u_flag = U_OFF;
 	goto reswitch;
@@ -271,7 +329,7 @@ REtest(char *str,		/* string to test */
 	if (s >= str_end)
 	    goto refill;
 	s++;
-	push(m, s, U_ON);
+	push(m, s, sp, U_ON);
 	m++;
 	u_flag = U_OFF;
 	goto reswitch;
@@ -321,19 +379,32 @@ REtest(char *str,		/* string to test */
 	m += m->s_data.jump;
 	goto reswitch;
 
+      CASE_UANY(M_SAVE_POS):	/* save position for a later M_2JC */
+	sp = RE_pos_push(sp, stackp, s);
+	m++;
+	goto reswitch;
+
       CASE_UANY(M_2JA):	/* take the non jump branch */
 	/* don't stack an ACCEPT */
 	if ((tm = m + m->s_data.jump)->s_type == M_ACCEPT)
 	    return 1;
-	push(tm, s, u_flag);
+	push(tm, s, sp, u_flag);
 	m++;
 	goto reswitch;
 
+      CASE_UANY(M_2JC):	/* take the jump branch if position changed */
+	if (RE_pos_pop(&sp, stackp) == s) {
+	    /* did not advance: do not jump back */
+	    m++;
+	    goto reswitch;
+	}
+	/* fall thru */
+
       CASE_UANY(M_2JB):	/* take the jump branch */
 	/* don't stack an ACCEPT */
 	if ((tm = m + 1)->s_type == M_ACCEPT)
 	    return 1;
-	push(tm, s, u_flag);
+	push(tm, s, sp, u_flag);
 	m += m->s_data.jump;
 	goto reswitch;
 
