@@ -12,7 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp3.c,v 1.27 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: rexp3.c,v 1.28 2013/08/03 14:04:00 tom Exp $
  * @Log: rexp3.c,v @
  * Revision 1.3  1993/07/24  17:55:15  mike
  * more cleanup
@@ -85,7 +85,6 @@ REmatch(char *str,		/* string to test */
     /* state of current best match stored here */
     char *cb_ss;		/* the start */
     char *cb_e = 0;		/* the end , pts at first char not matched */
-    STATE *m_best = 0;
 
     *lenp = 0;
 
@@ -431,7 +430,6 @@ REmatch(char *str,		/* string to test */
 	    /* we have a new current best */
 	    cb_ss = ss;
 	    cb_e = s;
-	    m_best = m;
 	}
 	goto refill;
 
@@ -446,7 +444,6 @@ REmatch(char *str,		/* string to test */
 	    /* we have a new current best */
 	    cb_ss = ss;
 	    cb_e = s;
-	    m_best = m;
 	}
 	goto refill;
 
