@@ -1,6 +1,7 @@
 /********************************************
 sizes.h
-copyright 1991, 1992.  Michael D. Brennan
+copyright 2009,2010.  Thomas E. Dickey
+copyright 1991-1994,1995.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: sizes.h,v 1.5 2009/07/26 19:11:23 tom Exp $
+ * $MawkId: sizes.h,v 1.7 2010/12/10 17:00:00 tom Exp $
  * @Log: sizes.h,v @
  * Revision 1.8  1995/10/14  22:09:51  mike
  * getting MAX__INT from values.h didn't really work since the value was
@@ -85,16 +86,29 @@ typedef unsigned UInt;
 #endif
 
 #define EVAL_STACK_SIZE  256	/* initial size , can grow */
-/* number of fields at startup, must be a power of 2
-   and FBANK_SZ-1 must be divisible by 3! */
-#define  FBANK_SZ	256
-#define  FB_SHIFT	  8	/* lg(FBANK_SZ) */
-#define  NUM_FBANK	128	/* see MAX_FIELD below */
+
+/*
+ * FBANK_SZ, the number of fields at startup, must be a power of 2.
+ * Also, FBANK_SZ-1 must be divisible by 3, since it is used for MAX_SPLIT.
+ *
+ * That means that FBANK_SZ can be 256, 1024, 4096, 16384, etc. (growing by
+ * a factor of four for each next possible value).
+ */
+#if 1
+#define  FBANK_SZ	 256
+#define  FB_SHIFT	   8	/* lg(FBANK_SZ) */
+#else
+#define  FBANK_SZ	1024
+#define  FB_SHIFT	  10	/* lg(FBANK_SZ) */
+#endif
+#define  NUM_FBANK	 128	/* see MAX_FIELD below */
 
 #define  MAX_SPLIT	(FBANK_SZ-1)	/* needs to be divisble by 3 */
 #define  MAX_FIELD	(NUM_FBANK*FBANK_SZ - 1)
-
-#define  MIN_SPRINTF	400
+/*
+ * mawk stores a union of MAX_SPLIT pointers and MIN_SPRINTF characters.
+ */
+#define  MIN_SPRINTF	(MAX_SPLIT * 4)
 
 #define  BUFFSZ         4096
   /* starting buffer size for input files, grows if
